#include <Adafruit_NeoPixel.h>

// Paramètres de la bande LED
#define LED_PIN 11         // Broche de données connectée à la bande LED
#define NUM_LEDS 28       // Nombre total de LEDs
#define BUTTON_PIN_1 8   // Bouton pour déclencher le CLIGNOTEMENT DROP (D10)
#define BUTTON_PIN_2 9   // Bouton pour CHANGEMENT DE COULEUR (D11)
#define BUTTON_PIN_3 10   // Bouton pour STROB (D12)

// Initialisation de l'objet NeoPixel
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

// Variables pour la gestion des boutons
int currentColorIndex = 0;         // Index de la couleur actuelle
unsigned long debounceDelay = 200; // Délai anti-rebond (ms)

// Couleurs prédéfinies
uint32_t colors[] = {
  0xFF0000,  // Rouge
  0x0000FF,  // Bleu
  0x00FF00,  // Vert
  0xFF00FF,  // Magenta
  0x000000   // Éteint
};
const int numColors = sizeof(colors) / sizeof(colors[0]); // Nombre de couleurs

void setup() {
  strip.begin();            // Initialise le ruban LED
  strip.show();             // Éteint toutes les LEDs au démarrage

  pinMode(BUTTON_PIN_1, INPUT);  // Bouton D10 avec pull-down
  pinMode(BUTTON_PIN_2, INPUT);  // Bouton D11 avec pull-down
  pinMode(BUTTON_PIN_3, INPUT);

  Serial.begin(9600);       // Pour débogage (facultatif)
}

void loop() {
  // Attendre un appui sur un bouton
  if (isButtonPressed(BUTTON_PIN_1)) {
    Serial.println("Bouton 1 appuyé, démarrage du clignotement !");
    clignoterLeds();  // Lancer le clignotement
  }

  if (isButtonPressed(BUTTON_PIN_2)) {
    Serial.println("Bouton 2 appuyé, changement de couleur !");
    changeColorCycle();  // Lancer le cycle de changement de couleurs
  }
  if (digitalRead(BUTTON_PIN_3) == HIGH) {
    Serial.println("Bouton 3 appuyé, Strobaumatic!");
    Strob();  
  }
}

// Fonction pour détecter un appui sur un bouton avec anti-rebond (logique inversée)
bool isButtonPressed(int buttonPin) {
  // Vérifie si le bouton est appuyé (état HIGH)
  if (digitalRead(buttonPin) == HIGH) {
    delay(debounceDelay); // Anti-rebond
    // Vérifie à nouveau si le bouton est toujours appuyé
    if (digitalRead(buttonPin) == HIGH) {
      // Attend que le bouton soit relâché avant de continuer (état LOW)
      while (digitalRead(buttonPin) == HIGH) {
        delay(10);
      }
      return true; // Confirme un appui validé
    }
  }
  return false; // Aucun appui détecté
}

// Fonction pour faire clignoter les LEDs en rouge
void clignoterLeds() {
  int delayTime = 500;
  int minDelay = 40;
  int decrement = 10;
  int rapidBlinkDuration = 4000;

  while (delayTime > minDelay) {
    // Allume les LEDs en rouge
    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, strip.Color(255, 0, 0));  // Rouge (R=255, G=0, B=0)
    }
    strip.show();
    delay(delayTime);

    // Éteint toutes les LEDs
    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, strip.Color(0, 0, 0));  // Éteint les LEDs
    }
    strip.show();
    delay(delayTime);

    // Réduction progressive du délai
    delayTime -= decrement;
  }

  // Maintenir le clignotement rapide pendant 5 secondes
  unsigned long startTime = millis();
  while (millis() - startTime < rapidBlinkDuration) {
    // Allume les LEDs
    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, strip.Color(255, 0, 0));  // Rouge
    }
    strip.show();
    delay(minDelay);

    // Éteint les LEDs
    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, strip.Color(0, 0, 0));  // Éteint les LEDs
    }
    strip.show();
    delay(minDelay);
  }

  // Éteindre les LEDs après le clignotement
  strip.clear();
  strip.show();
  Serial.println("Clignotement terminé, LEDs éteintes.");
}

// Fonction pour changer la couleur des LEDs
void changeColorCycle() {
  while (true) {
    // Avancer dans le cycle des couleurs
    currentColorIndex = (currentColorIndex + 1) % numColors;

    // Appliquer la couleur correspondante
    uint32_t color = colors[currentColorIndex];
    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, color);
    }
    strip.show();

    // Afficher dans le moniteur série la couleur actuelle
    switch (currentColorIndex) {
      case 0: Serial.println("Couleur actuelle : Rouge"); break;
      case 1: Serial.println("Couleur actuelle : Bleu"); break;
      case 2: Serial.println("Couleur actuelle : Vert"); break;
      case 3: Serial.println("Couleur actuelle : Magenta"); break;
      case 4: 
        Serial.println("Couleur actuelle : Éteint");
        strip.clear();  // Éteint toutes les LEDs
        strip.show();
        Serial.println("Cycle terminé, LEDs éteintes. En attente d'un nouveau bouton.");
        return;  // Quitter la fonction pour revenir à loop()
    }

    // Attente d'un nouvel appui sur le bouton pour passer à la couleur suivante
    while (true) {
      if (isButtonPressed(BUTTON_PIN_2)) {
        break; // Passer à la prochaine couleur
      }
    }
  }
}

void Strob() {
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(255, 255, 255));  // Définit la LED i en blanc
    strip.show();                                   // Met à jour l'affichage des LEDs
  }
  delay(50);  // fréquence du strobe
  // Éteindre toutes les LEDs pour recommencer l'animation
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 0));   // Éteint la LED i
    strip.show();                     // Met à jour l'affichage des LEDs
  }
  delay(50);  // fréquence du strobe
}


